#macでは完成
import os
import cv2
import configparser
import errno
import numpy as np

#config.ini読み込み
config_ini = configparser.ConfigParser()
config_ini_path = 'config.ini'
#指定したファイルが存在しない場合、エラー発生
if not os.path.exists(config_ini_path):
   raise FileNotFoundError(errno.ENOENT, os.strerror(errno.ENOENT), config_ini_path)

config_ini.read(config_ini_path,encoding='utf-8')

#iniの値取得
#パスとフィルタの強さ
read_default = config_ini['DEFAULT']
var = read_default.get('PATH')
folder_path = var
print('フォルダパス:', var)

# var2 = read_default.get('STRENGTH')
# strength = int(var2)
# print('フィルタの強さ:',var2)

text = '''
>フィルタの強さを変更するとぼやけている画像の判定が変わります
>フィルタの強さを変更する際は1ずつ変更を加えるといいかもしれません
>参考数値:10,11,12,13,14,15,16 ※iphoneで撮影した場合の参考 環境によっては異なります
'''

while True:
    try:
        print(text)
        strength = input('フィルタの強さの設定値(数値)を入力:')
        #print(strength)
        if strength.isdecimal():
            strength = int(strength)
            break
        else:
            pass
        
    except ValueError:
        print('Invalid input!')




#画像があるフォルダパスを指定
#20240120 tiff raw追加
image_files = [f for f in os.listdir(folder_path) if f.lower().endswith(('.jpg', '.png', '.gif','.tiff','.raw'))]

# ファイル名を数字順にソート
sorted_image_files = sorted(image_files, key=lambda x: int(''.join(filter(str.isdigit, x))))



for image_file in sorted_image_files:
    print(os.path.join(folder_path, image_file))

###################################################


    #image = cv2.imread(image_path,flags=cv2.IMREAD_COLOR,dtype = np.uint8)
    #日本語文字化け対策
def imread(filename, flags=cv2.IMREAD_COLOR, dtype=np.uint8):
    try:
        n = np.fromfile(filename, dtype)
        img = cv2.imdecode(n, flags)
        return img
    except Exception as e:
        print(e)
        return None

        
#threshold: ぼけを判定する閾値です。
#デフォルト値は100ですが、この値を変更することでぼけの判定基準を変更

def is_blurred(image_path, threshold=10):
    if not os.path.exists(image_path):
        print(f"画像 '{image_path}' は存在しません。")
        return None
    

    image = imread(image_path)

    if image is None:
        print(f"画像 '{image_path}' を読み込めませんでした。")
        return None

    # ↓画像をグレースケールに変換
    gray = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)

    # ↓ガウシアンフィルタを適用してノイズを平滑化
    blurred = cv2.GaussianBlur(gray, (5, 5), 0)
    
    # ↓ラプラシアンを計算
    laplacian = cv2.Laplacian(blurred, cv2.CV_64F)

    # -ラプラシアンの分散を計算
    variance = laplacian.var()

    # 分散が閾値未満であればぼやけていると判断
    return variance < threshold
###################################################

#ここのthreshold変更
#iphoneは16がいいかも?ハード依存かは調査中
def detect_blur_in_images(sorted_image_files, threshold=strength):
    results = {}
    
    # print(sorted_image_files)

    for image_path in sorted_image_files:
        blur_result = is_blurred(os.path.join(folder_path, image_path), threshold)
        results[os.path.join(folder_path, image_path)] = blur_result

    return results
###################################################

# ぼけているかどうかを判定し表示
blur_results = detect_blur_in_images(sorted_image_files)

#NG用のパス
new_directory_path = os.path.join(folder_path, "NG")
print(new_directory_path)

#ぼやけている画像用フォルダなければ作成
if not os.path.exists(new_directory_path):
    os.mkdir(new_directory_path)

#画像のNGの数をカウントする変数宣言、初期化
cnt_ng = 0
cnt_all = len(image_files)

# 結果を表示 blurはぼかしという意味
for image_path, blur_result in blur_results.items():
    if blur_result is not None:
        if blur_result:
            print(f"画像 '{image_path}' はぼやけています。")
            cnt_ng += 1

            try:
             os.rename(image_path,os.path.join(new_directory_path,
                       os.path.basename(image_path)))
            except Exception as e:
                print(f"NGフォルダへのファイルの移動中にエラーが発生しました:{e}")
        else:
            print(f"画像 '{image_path}' はぼやけていません。")
            
#NG画像が全体でいくつあったか出力
print(f"NG画像(ぼやけた画像)の割合:{cnt_ng}/{cnt_all}")

#NG判定のフォルダOKかユーザーに問いかける、noを言われたら元に戻す
while True:
    #ValueErrorは引き渡す値が間違っている場合でるエラー
    try:
        val_in = input('NGフォルダ内の画像はNGですか? noを入力すると元に戻します(y/n):').lower()
       
        if val_in == "y" or val_in == "yes":
            break

        elif val_in == "n" or val_in == "no":
            for filename in os.listdir(new_directory_path):
                source_path = os.path.join(new_directory_path, filename)
                destination_path = os.path.join(folder_path, filename)
                try:
                    os.rename(source_path,destination_path)
                except Exception as e:
                    print(f"元のフォルダへのファイルの移動中にエラーが発生しました:{e}")

            try:
                os.rmdir(new_directory_path)
            except OSError as e:
                print(f"Error deleting folder '{new_directory_path}':{e}")

            break 

        else:
            pass

    #except の隣に発生し得るエラーを記述する
    except ValueError:
        print('Invalid input!')






###################################################
# from kivy.app import App
# from kivy.uix.boxlayout import BoxLayout
# from kivy.uix.button import Button
# from kivy.uix.filechooser import FileChooserListView

# class FileChooserApp(App):
#     def build(self):
#         layout = BoxLayout(orientation='vertical', spacing=10, padding=10)

#         # ファイル選択ダイアログ
#         file_chooser = FileChooserListView()
#         layout.add_widget(file_chooser)

#         # ファイルを選択したときのボタン
#         select_button = Button(text="選択したファイルを表示", on_press=self.show_selected_file)
#         layout.add_widget(select_button)

#         return layout

#     def show_selected_file(self, instance):
#         # 選択されたファイルのパスを取得
#         selected_file = self.root.children[0].selection and self.root.children[0].selection[0] or None

#         if selected_file:
#             print("選択されたファイル:", selected_file)
#         else:
#             print("ファイルが選択されていません。")

# if __name__ == '__main__':
#     FileChooserApp().run()